@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 2021 LunarG, Inc.
+** Copyright (c) 2021-2023 LunarG, Inc.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -38,21 +38,16 @@ Dx12AsciiConsumerBase::~Dx12AsciiConsumerBase()
 void Dx12AsciiConsumerBase::Initialize(FILE* file, gfxrecon::util::ToStringFlags toStringFlags)
 {
     assert(file);
-    file_ = file;
-    if (to_string_flags_ & gfxrecon::util::kToString_Formatted)
-    {
-        fprintf(file_, "{\n\"apiCalls\":[");
-    }
+    file_            = file;
+    to_string_flags_ = toStringFlags;
+    // Note the Vulkan Consumer will top and tail the file with
+    // a JSON array if output is not JSONL.
 }
 
 void Dx12AsciiConsumerBase::Destroy()
 {
     if (file_)
     {
-        if (to_string_flags_ & gfxrecon::util::kToString_Formatted)
-        {
-            fprintf(file_, "\n]\n}\n");
-        }
         file_ = nullptr;
     }
 }
