@@ -1,6 +1,6 @@
 /*
 ** Copyright (c) 2022 LunarG, Inc.
-** Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+** Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -80,7 +80,9 @@ void GetRootSignatureResourceValueInfos(const T* root_signature_desc, std::set<R
                 auto aligned_offset = util::platform::AlignValue<sizeof(D3D12_GPU_DESCRIPTOR_HANDLE::ptr)>(byte_offset);
                 value_infos.insert({ aligned_offset,
                                      ResourceValueType::kGpuDescriptorHandle,
-                                     sizeof(D3D12_GPU_DESCRIPTOR_HANDLE::ptr) });
+                                     sizeof(D3D12_GPU_DESCRIPTOR_HANDLE::ptr),
+                                     nullptr,
+                                     { nullptr, nullptr, 0 } });
                 byte_offset = aligned_offset + sizeof(D3D12_GPU_DESCRIPTOR_HANDLE::ptr);
             }
             break;
@@ -89,8 +91,11 @@ void GetRootSignatureResourceValueInfos(const T* root_signature_desc, std::set<R
             case D3D12_ROOT_PARAMETER_TYPE_UAV:
             {
                 auto aligned_offset = util::platform::AlignValue<sizeof(D3D12_GPU_VIRTUAL_ADDRESS)>(byte_offset);
-                value_infos.insert(
-                    { aligned_offset, ResourceValueType::kGpuVirtualAddress, sizeof(D3D12_GPU_VIRTUAL_ADDRESS) });
+                value_infos.insert({ aligned_offset,
+                                     ResourceValueType::kGpuVirtualAddress,
+                                     sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
+                                     nullptr,
+                                     { nullptr, nullptr, 0 } });
                 byte_offset = aligned_offset + sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
             }
             break;
@@ -112,7 +117,11 @@ void CopyResourceValuesFromDstToSrc(std::set<ResourceValueInfo>&          src,
     for (auto dst_iter = dst_begin; dst_iter != dst_end; ++dst_iter)
     {
         auto src_offset = ((*dst_iter).offset - copy_info.dst_offset) + copy_info.src_offset;
-        src.insert({ src_offset, (*dst_iter).type, (*dst_iter).size, (*dst_iter).state_object });
+        src.insert({ static_cast<uint64_t>(src_offset),
+                     (*dst_iter).type,
+                     static_cast<uint64_t>((*dst_iter).size),
+                     (*dst_iter).state_object,
+                     (*dst_iter).arg_buffer_extra_info });
     }
 }
 
@@ -366,13 +375,17 @@ void Dx12ResourceValueMapper::PostProcessCreateCommandSignature(HandlePointerDec
             case D3D12_INDIRECT_ARGUMENT_TYPE_VERTEX_BUFFER_VIEW:
                 resource_value_infos.insert({ byte_offset + offsetof(D3D12_VERTEX_BUFFER_VIEW, BufferLocation),
                                               ResourceValueType::kGpuVirtualAddress,
-                                              sizeof(D3D12_VERTEX_BUFFER_VIEW::BufferLocation) });
+                                              sizeof(D3D12_VERTEX_BUFFER_VIEW::BufferLocation),
+                                              nullptr,
+                                              { nullptr, nullptr, 0 } });
                 byte_offset += sizeof(D3D12_VERTEX_BUFFER_VIEW);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_INDEX_BUFFER_VIEW:
                 resource_value_infos.insert({ byte_offset + offsetof(D3D12_INDEX_BUFFER_VIEW, BufferLocation),
                                               ResourceValueType::kGpuVirtualAddress,
-                                              sizeof(D3D12_INDEX_BUFFER_VIEW::BufferLocation) });
+                                              sizeof(D3D12_INDEX_BUFFER_VIEW::BufferLocation),
+                                              nullptr,
+                                              { nullptr, nullptr, 0 } });
                 byte_offset += sizeof(D3D12_INDEX_BUFFER_VIEW);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_CONSTANT:
@@ -381,14 +394,19 @@ void Dx12ResourceValueMapper::PostProcessCreateCommandSignature(HandlePointerDec
             case D3D12_INDIRECT_ARGUMENT_TYPE_CONSTANT_BUFFER_VIEW:
             case D3D12_INDIRECT_ARGUMENT_TYPE_SHADER_RESOURCE_VIEW:
             case D3D12_INDIRECT_ARGUMENT_TYPE_UNORDERED_ACCESS_VIEW:
-                resource_value_infos.insert(
-                    { byte_offset, ResourceValueType::kGpuVirtualAddress, sizeof(D3D12_GPU_VIRTUAL_ADDRESS) });
+                resource_value_infos.insert({ static_cast<uint64_t>(byte_offset),
+                                              ResourceValueType::kGpuVirtualAddress,
+                                              sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
+                                              nullptr,
+                                              { nullptr, nullptr, 0 } });
                 byte_offset += sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_RAYS:
-                resource_value_infos.insert({ byte_offset,
+                resource_value_infos.insert({ static_cast<uint64_t>(byte_offset),
                                               ResourceValueType::kIndirectArgumentDispatchRays,
-                                              sizeof(D3D12_DISPATCH_RAYS_DESC) });
+                                              sizeof(D3D12_DISPATCH_RAYS_DESC),
+                                              nullptr,
+                                              { nullptr, nullptr, 0 } });
                 byte_offset += sizeof(D3D12_DISPATCH_RAYS_DESC);
                 break;
             case D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_MESH:
@@ -415,32 +433,30 @@ void Dx12ResourceValueMapper::PostProcessExecuteIndirect(DxObjectInfo* command_l
     auto command_signature_extra_info = GetExtraInfo<D3D12CommandSignatureInfo>(command_signature_object_info);
     GFXRECON_ASSERT(command_signature_extra_info != nullptr);
 
-    auto& resource_value_infos = command_list_extra_info->resource_value_info_map[argument_buffer_object_info];
-
-    // TODO: Support pCountBuffer argument.
-    if ((count_buffer_object_info != nullptr) && (count_buffer_object_info->object != nullptr))
-    {
-        GFXRECON_LOG_WARNING_ONCE("The pCountBuffer argument for ExecuteIndirect is not currently supported. The "
-                                  "contents of pArgumentBuffer may become corrupted.");
-    }
-
-    // Add resource value offsets to resource_value_infos based on the command signature's arguments.
     D3D12StateObjectInfo* state_object_extra_info = nullptr;
     if (command_list_extra_info->active_state_object != nullptr)
     {
         state_object_extra_info = GetExtraInfo<D3D12StateObjectInfo>(command_list_extra_info->active_state_object);
     }
-    uint64_t command_byte_offset = argument_buffer_offset;
-    for (UINT i = 0; i < max_command_count; ++i)
+
+    if ((count_buffer_object_info != nullptr) && (count_buffer_object_info->object != nullptr))
     {
-        for (const auto& resource_value_info : command_signature_extra_info->resource_value_infos)
-        {
-            resource_value_infos.insert({ resource_value_info.offset + command_byte_offset,
-                                          resource_value_info.type,
-                                          resource_value_info.size,
-                                          state_object_extra_info });
-        }
-        command_byte_offset += command_signature_extra_info->byte_stride;
+        auto& resource_value_infos = command_list_extra_info->resource_value_info_map[count_buffer_object_info];
+        resource_value_infos.insert(
+            { count_buffer_offset,
+              ResourceValueType::kExecuteIndirectCountBuffer,
+              sizeof(uint32_t),
+              state_object_extra_info,
+              { command_signature_extra_info, argument_buffer_object_info, argument_buffer_offset } });
+    }
+    else
+    {
+        // Add resource value offsets to resource_value_infos based on the command signature's arguments.
+        GetExecuteIndirectResourceValues(command_list_extra_info->resource_value_info_map[argument_buffer_object_info],
+                                         command_signature_extra_info->resource_value_infos,
+                                         max_command_count,
+                                         argument_buffer_offset,
+                                         command_signature_extra_info->byte_stride);
     }
 }
 
@@ -455,16 +471,22 @@ void Dx12ResourceValueMapper::PostProcessBuildRaytracingAccelerationStructure(
         GFXRECON_ASSERT(build_desc->Inputs.InstanceDescs != 0);
 
         format::HandleId resource_id = format::kNullHandleId;
-        reverse_gpu_va_map_.Map(build_desc->Inputs.InstanceDescs, &resource_id);
+        bool             found       = false;
+        reverse_gpu_va_map_.Map(build_desc->Inputs.InstanceDescs,
+                                &resource_id,
+                                &found,
+                                build_desc->Inputs.InstanceDescs +
+                                    build_desc->Inputs.NumDescs * sizeof(D3D12_RAYTRACING_INSTANCE_DESC));
+
         if (resource_id != format::kNullHandleId)
         {
-            auto resouce_object_info = get_object_info_func_(resource_id);
-            GFXRECON_ASSERT(resouce_object_info != nullptr);
-            GFXRECON_ASSERT(resouce_object_info->object != nullptr);
+            auto resource_object_info = get_object_info_func_(resource_id);
+            GFXRECON_ASSERT(resource_object_info != nullptr);
+            GFXRECON_ASSERT(resource_object_info->object != nullptr);
 
-            auto  resource                = static_cast<ID3D12Resource*>(resouce_object_info->object);
+            auto  resource                = static_cast<ID3D12Resource*>(resource_object_info->object);
             auto  command_list_extra_info = GetExtraInfo<D3D12CommandListInfo>(command_list4_object_info);
-            auto& resource_value_infos    = command_list_extra_info->resource_value_info_map[resouce_object_info];
+            auto& resource_value_infos    = command_list_extra_info->resource_value_info_map[resource_object_info];
 
             GFXRECON_ASSERT(build_desc->Inputs.InstanceDescs >= resource->GetGPUVirtualAddress());
             auto offset_to_instance_descs_start = build_desc->Inputs.InstanceDescs - resource->GetGPUVirtualAddress();
@@ -482,7 +504,9 @@ void Dx12ResourceValueMapper::PostProcessBuildRaytracingAccelerationStructure(
                     resource_value_infos.insert(
                         { offset_to_instance_descs_start + accel_struct_gpu_va_stride * i + accel_struct_gpu_va_offset,
                           ResourceValueType::kGpuVirtualAddress,
-                          sizeof(D3D12_GPU_VIRTUAL_ADDRESS) });
+                          sizeof(D3D12_GPU_VIRTUAL_ADDRESS),
+                          nullptr,
+                          { nullptr, nullptr, 0 } });
                 }
             }
             else
@@ -835,8 +859,13 @@ void Dx12ResourceValueMapper::CopyResourceValues(const ResourceCopyInfo& copy_in
         // If num_bytes != 0, process CopyBufferRegion (partial copy) else process CopyResource (full copy).
         if (copy_info.num_bytes != 0)
         {
-            auto dst_values_begin = dst_values.lower_bound({ copy_info.dst_offset });
-            auto dst_values_end   = dst_values.upper_bound({ copy_info.dst_offset + copy_info.num_bytes });
+            auto dst_values_begin = dst_values.lower_bound(
+                { copy_info.dst_offset, ResourceValueType::kUnknown, 0, nullptr, { nullptr, nullptr, 0 } });
+            auto dst_values_end = dst_values.upper_bound({ copy_info.dst_offset + copy_info.num_bytes,
+                                                           ResourceValueType::kUnknown,
+                                                           0,
+                                                           nullptr,
+                                                           { nullptr, nullptr, 0 } });
             CopyResourceValuesFromDstToSrc(src_values, dst_values, dst_values_begin, dst_values_end, copy_info);
             dst_values.erase(dst_values_begin, dst_values_end);
         }
@@ -1029,7 +1058,7 @@ bool Dx12ResourceValueMapper::IsNonEmptyShaderRecord(const std::vector<uint8_t>&
     return is_empty;
 }
 
-void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
+bool Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
                                        std::vector<uint8_t>&    result_data,
                                        format::HandleId         resource_id,
                                        D3D12ResourceInfo*       resource_info,
@@ -1047,7 +1076,7 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
 
         if (current_address == 0)
         {
-            return;
+            return false;
         }
 
         auto mapped_value_iter = resource_info->mapped_gpu_addresses.find(final_offset);
@@ -1064,7 +1093,7 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
         {
             if (current_address == mapped_value_iter->second)
             {
-                return;
+                return false;
             }
         }
 
@@ -1099,6 +1128,7 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
                 resource_info->mapped_gpu_addresses[final_offset] = *address;
             }
         }
+        return true;
     }
     else if (value_info.type == ResourceValueType::kShaderIdentifier)
     {
@@ -1115,7 +1145,7 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
         // Don't attempt to map shader records with shader ID == 0.
         if (current_id == zero_id)
         {
-            return;
+            return false;
         }
 
         auto mapped_shader_id_iter = resource_info->mapped_shader_ids.find(final_offset);
@@ -1136,7 +1166,7 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
                 // It is possible for a shader to specify a multiplier (e.g.,
                 // MultiplierForGeometryContributionToHitGroupIndex) for the shader record stride so not all data in the
                 // range of the shader table is necessarily a shader record.
-                return;
+                return false;
             }
         }
 
@@ -1187,6 +1217,7 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
                 }
             }
         }
+        return true;
     }
     else if (value_info.type == ResourceValueType::kIndirectArgumentDispatchRays)
     {
@@ -1195,7 +1226,7 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
         uint8_t* desc_data_ptr = result_data.data() + final_offset;
 
         // First map the StartAddress GPU VAs from D3D12_DISPATCH_RAYS_DESC in resource data.
-        ResourceValueInfo rvi;
+        ResourceValueInfo rvi{ 0, ResourceValueType::kUnknown, 0, nullptr, { nullptr, nullptr, 0 } };
         rvi.size         = sizeof(D3D12_GPU_VIRTUAL_ADDRESS);
         rvi.type         = ResourceValueType::kGpuVirtualAddress;
         rvi.state_object = value_info.state_object;
@@ -1226,11 +1257,34 @@ void Dx12ResourceValueMapper::MapValue(const ResourceValueInfo& value_info,
         GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, value_info.size);
         util::platform::MemoryCopy(&desc, sizeof(desc), desc_data_ptr, static_cast<size_t>(value_info.size));
         GetDispatchRaysResourceValues(indirect_values_map, value_info.state_object, desc);
+        return true;
+    }
+    else if (value_info.type == ResourceValueType::kExecuteIndirectCountBuffer)
+    {
+        // Read command counts while CountBuffer contents is still visible in temp_resource_data
+        uint32_t command_count = 0;
+        util::platform::MemoryCopy(&command_count,
+                                   sizeof(command_count),
+                                   temp_resource_data.data() + value_info.offset,
+                                   sizeof(command_count));
+
+        // Insert new ArgumentBuffer RV infos, which will queue them for translation.
+        if (command_count != 0)
+        {
+            GetExecuteIndirectResourceValues(
+                indirect_values_map[value_info.arg_buffer_extra_info.argument_buffer],
+                value_info.arg_buffer_extra_info.command_signature_info->resource_value_infos,
+                command_count,
+                value_info.arg_buffer_extra_info.argument_buffer_offset,
+                value_info.arg_buffer_extra_info.command_signature_info->byte_stride);
+        }
+        return false;
     }
     else
     {
         GFXRECON_ASSERT(false && "Unrecognized resource value type.");
     }
+    return false;
 }
 
 void Dx12ResourceValueMapper::MapResources(const ResourceValueInfoMap&                        resource_value_info_map,
@@ -1287,28 +1341,35 @@ void Dx12ResourceValueMapper::MapResources(const ResourceValueInfoMap&
             auto resource_extra_info = GetResourceExtraInfo(resource_object_info);
 
             MappedResourceRevertInfo revert_info;
-            revert_info.data                                    = temp_resource_data;
-            revert_info.states                                  = temp_resource_states;
-            revert_info.mapped_gpu_addresses                    = resource_extra_info->mapped_gpu_addresses;
-            revert_info.mapped_shader_ids                       = resource_extra_info->mapped_shader_ids;
-            resource_data_to_revert[resource_value_infos.first] = std::move(revert_info);
+            revert_info.data                 = temp_resource_data;
+            revert_info.states               = temp_resource_states;
+            revert_info.mapped_gpu_addresses = resource_extra_info->mapped_gpu_addresses;
+            revert_info.mapped_shader_ids    = resource_extra_info->mapped_shader_ids;
 
+            bool write_back = false;
             for (const auto& value_info : value_infos)
             {
-                MapValue(value_info,
-                         temp_resource_data,
-                         resource_object_info->capture_id,
-                         resource_extra_info,
-                         indirect_values_map);
+                if (MapValue(value_info,
+                             temp_resource_data,
+                             resource_object_info->capture_id,
+                             resource_extra_info,
+                             indirect_values_map) == true)
+                {
+                    write_back = true;
+                }
             }
 
-            hr = resource_data_util_->WriteToResource(resource,
-                                                      true,
-                                                      temp_resource_states,
-                                                      temp_resource_states,
-                                                      temp_resource_data,
-                                                      temp_resource_sizes,
-                                                      temp_resource_offsets);
+            if (write_back == true)
+            {
+                resource_data_to_revert[resource_value_infos.first] = std::move(revert_info);
+                hr                                                  = resource_data_util_->WriteToResource(resource,
+                                                          true,
+                                                          temp_resource_states,
+                                                          temp_resource_states,
+                                                          temp_resource_data,
+                                                          temp_resource_sizes,
+                                                          temp_resource_offsets);
+            }
 
             if (FAILED(hr))
             {
@@ -1366,7 +1427,8 @@ void Dx12ResourceValueMapper::GetShaderTableResourceValues(ResourceValueInfoMap&
         return;
     }
     format::HandleId resource_id = format::kNullHandleId;
-    reverse_gpu_va_map_.Map(start_address, &resource_id);
+    bool             found       = false;
+    reverse_gpu_va_map_.Map(start_address, &resource_id, &found, start_address + size);
     if (resource_id == format::kNullHandleId)
     {
         return;
@@ -1376,11 +1438,11 @@ void Dx12ResourceValueMapper::GetShaderTableResourceValues(ResourceValueInfoMap&
     GFXRECON_ASSERT((start_address % D3D12_RAYTRACING_SHADER_TABLE_BYTE_ALIGNMENT) == 0);
     GFXRECON_ASSERT((stride % D3D12_RAYTRACING_SHADER_RECORD_BYTE_ALIGNMENT) == 0);
 
-    auto resouce_object_info = get_object_info_func_(resource_id);
-    GFXRECON_ASSERT((resouce_object_info != nullptr) && (resouce_object_info->object != nullptr));
-    auto resource = static_cast<ID3D12Resource*>(resouce_object_info->object);
+    auto resource_object_info = get_object_info_func_(resource_id);
+    GFXRECON_ASSERT((resource_object_info != nullptr) && (resource_object_info->object != nullptr));
+    auto resource = static_cast<ID3D12Resource*>(resource_object_info->object);
 
-    auto& resource_value_infos = resource_value_info_map[resouce_object_info];
+    auto& resource_value_infos = resource_value_info_map[resource_object_info];
 
     UINT64 shader_record_count = 1;
     UINT64 shader_record_size  = size;
@@ -1394,8 +1456,11 @@ void Dx12ResourceValueMapper::GetShaderTableResourceValues(ResourceValueInfoMap&
     GFXRECON_ASSERT((byte_offset % D3D12_RAYTRACING_SHADER_TABLE_BYTE_ALIGNMENT) == 0);
     for (UINT64 i = 0; i < shader_record_count; ++i)
     {
-        resource_value_infos.insert(
-            { byte_offset, ResourceValueType::kShaderIdentifier, shader_record_size, state_object_extra_info });
+        resource_value_infos.insert({ byte_offset,
+                                      ResourceValueType::kShaderIdentifier,
+                                      shader_record_size,
+                                      state_object_extra_info,
+                                      { nullptr, nullptr, 0 } });
         byte_offset += shader_record_size;
     }
 }
@@ -1434,6 +1499,27 @@ void Dx12ResourceValueMapper::GetDispatchRaysResourceValues(ResourceValueInfoMap
                                  desc.CallableShaderTable.StrideInBytes);
 }
 
+void Dx12ResourceValueMapper::GetExecuteIndirectResourceValues(
+    std::set<ResourceValueInfo>& dst_resource_value_info_map,
+    std::set<ResourceValueInfo>& command_signature_resource_value_info_map,
+    uint32_t                     command_count,
+    uint64_t                     command_offset,
+    uint8_t                      stride)
+{
+    for (uint32_t i = 0; i < command_count; ++i)
+    {
+        for (const auto& resource_value_info : command_signature_resource_value_info_map)
+        {
+            dst_resource_value_info_map.insert({ resource_value_info.offset + command_offset,
+                                                 resource_value_info.type,
+                                                 resource_value_info.size,
+                                                 resource_value_info.state_object,
+                                                 resource_value_info.arg_buffer_extra_info });
+        }
+        command_offset += stride;
+    }
+}
+
 void Dx12ResourceValueMapper::GetStateObjectLrsAssociationInfo(
     format::HandleId                                       state_object_id,
     StructPointerDecoder<Decoded_D3D12_STATE_OBJECT_DESC>* desc_decoder,
