@@ -34,9 +34,10 @@ VulkanReferencedResourceConsumerBase::VulkanReferencedResourceConsumerBase() :
     loading_state_(false), loaded_state_(false)
 {}
 
-void VulkanReferencedResourceConsumerBase::Process_vkQueueSubmit(VkResult         returnValue,
-                                                                 format::HandleId queue,
-                                                                 uint32_t         submitCount,
+void VulkanReferencedResourceConsumerBase::Process_vkQueueSubmit(const ApiCallInfo& call_info,
+                                                                 VkResult           returnValue,
+                                                                 format::HandleId   queue,
+                                                                 uint32_t           submitCount,
                                                                  StructPointerDecoder<Decoded_VkSubmitInfo>* pSubmits,
                                                                  format::HandleId                            fence)
 {
@@ -65,7 +66,40 @@ void VulkanReferencedResourceConsumerBase::Process_vkQueueSubmit(VkResult
     }
 }
 
+void VulkanReferencedResourceConsumerBase::Process_vkQueueSubmit2(const ApiCallInfo& call_info,
+                                                                  VkResult           returnValue,
+                                                                  format::HandleId   queue,
+                                                                  uint32_t           submitCount,
+                                                                  StructPointerDecoder<Decoded_VkSubmitInfo2>* pSubmits,
+                                                                  format::HandleId                             fence)
+{
+    GFXRECON_UNREFERENCED_PARAMETER(returnValue);
+    GFXRECON_UNREFERENCED_PARAMETER(queue);
+    GFXRECON_UNREFERENCED_PARAMETER(submitCount);
+    GFXRECON_UNREFERENCED_PARAMETER(fence);
+
+    assert(pSubmits != nullptr);
+
+    if (!pSubmits->IsNull() && pSubmits->HasData())
+    {
+        size_t     submit_count = pSubmits->GetLength();
+        const auto submits      = pSubmits->GetMetaStructPointer();
+
+        for (size_t i = 0; i < submit_count; ++i)
+        {
+            size_t     command_buffer_count = submits[i].pCommandBufferInfos->GetLength();
+            const auto command_buffers      = submits[i].pCommandBufferInfos->GetMetaStructPointer();
+
+            for (size_t j = 0; j < command_buffer_count; ++j)
+            {
+                table_.ProcessUserSubmission(command_buffers[j].commandBuffer);
+            }
+        }
+    }
+}
+
 void VulkanReferencedResourceConsumerBase::Process_vkCreateBuffer(
+    const ApiCallInfo&                                   call_info,
     VkResult                                             returnValue,
     format::HandleId                                     device,
     StructPointerDecoder<Decoded_VkBufferCreateInfo>*    pCreateInfo,
@@ -100,6 +134,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateBuffer(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCreateBufferView(
+    const ApiCallInfo&                                    call_info,
     VkResult                                              returnValue,
     format::HandleId                                      device,
     StructPointerDecoder<Decoded_VkBufferViewCreateInfo>* pCreateInfo,
@@ -120,6 +155,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateBufferView(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCreateImage(
+    const ApiCallInfo&                                   call_info,
     VkResult                                             returnValue,
     format::HandleId                                     device,
     StructPointerDecoder<Decoded_VkImageCreateInfo>*     pCreateInfo,
@@ -154,6 +190,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateImage(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCreateImageView(
+    const ApiCallInfo&                                   call_info,
     VkResult                                             returnValue,
     format::HandleId                                     device,
     StructPointerDecoder<Decoded_VkImageViewCreateInfo>* pCreateInfo,
@@ -174,6 +211,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateImageView(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCreateFramebuffer(
+    const ApiCallInfo&                                     call_info,
     VkResult                                               returnValue,
     format::HandleId                                       device,
     StructPointerDecoder<Decoded_VkFramebufferCreateInfo>* pCreateInfo,
@@ -197,6 +235,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateFramebuffer(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCreateDescriptorSetLayout(
+    const ApiCallInfo&                                             call_info,
     VkResult                                                       returnValue,
     format::HandleId                                               device,
     StructPointerDecoder<Decoded_VkDescriptorSetLayoutCreateInfo>* pCreateInfo,
@@ -224,6 +263,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateDescriptorSetLayout(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCreateDescriptorUpdateTemplate(
+    const ApiCallInfo&                                                  call_info,
     VkResult                                                            returnValue,
     format::HandleId                                                    device,
     StructPointerDecoder<Decoded_VkDescriptorUpdateTemplateCreateInfo>* pCreateInfo,
@@ -238,6 +278,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateDescriptorUpdateTempl
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCreateDescriptorUpdateTemplateKHR(
+    const ApiCallInfo&                                                  call_info,
     VkResult                                                            returnValue,
     format::HandleId                                                    device,
     StructPointerDecoder<Decoded_VkDescriptorUpdateTemplateCreateInfo>* pCreateInfo,
@@ -252,6 +293,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCreateDescriptorUpdateTempl
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkDestroyDescriptorPool(
+    const ApiCallInfo&                                   call_info,
     format::HandleId                                     device,
     format::HandleId                                     descriptorPool,
     StructPointerDecoder<Decoded_VkAllocationCallbacks>* pAllocator)
@@ -262,7 +304,8 @@ void VulkanReferencedResourceConsumerBase::Process_vkDestroyDescriptorPool(
     table_.ClearContainers(descriptorPool);
 }
 
-void VulkanReferencedResourceConsumerBase::Process_vkResetDescriptorPool(VkResult                   returnValue,
+void VulkanReferencedResourceConsumerBase::Process_vkResetDescriptorPool(const ApiCallInfo&         call_info,
+                                                                         VkResult                   returnValue,
                                                                          format::HandleId           device,
                                                                          format::HandleId           descriptorPool,
                                                                          VkDescriptorPoolResetFlags flags)
@@ -275,6 +318,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkResetDescriptorPool(VkResul
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkAllocateDescriptorSets(
+    const ApiCallInfo&                                         call_info,
     VkResult                                                   returnValue,
     format::HandleId                                           device,
     StructPointerDecoder<Decoded_VkDescriptorSetAllocateInfo>* pAllocateInfo,
@@ -307,6 +351,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkAllocateDescriptorSets(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkFreeDescriptorSets(
+    const ApiCallInfo&                     call_info,
     VkResult                               returnValue,
     format::HandleId                       device,
     format::HandleId                       descriptorPool,
@@ -331,6 +376,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkFreeDescriptorSets(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSets(
+    const ApiCallInfo&                                  call_info,
     format::HandleId                                    device,
     uint32_t                                            descriptorWriteCount,
     StructPointerDecoder<Decoded_VkWriteDescriptorSet>* pDescriptorWrites,
@@ -360,7 +406,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSets(
                 case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE:
                 case VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT:
                 {
-                    const auto image_info = meta_writes->pImageInfo;
+                    const auto image_info = meta_writes[i].pImageInfo;
                     assert(image_info != nullptr);
 
                     if (!image_info->IsNull() && image_info->HasData())
@@ -379,7 +425,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSets(
                 case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
                 case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC:
                 {
-                    const auto buffer_info = meta_writes->pBufferInfo;
+                    const auto buffer_info = meta_writes[i].pBufferInfo;
                     assert(buffer_info != nullptr);
 
                     if (!buffer_info->IsNull() && buffer_info->HasData())
@@ -396,7 +442,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSets(
                 case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
                 case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
                 {
-                    const auto& views = meta_writes->pTexelBufferView;
+                    const auto& views = meta_writes[i].pTexelBufferView;
 
                     if (!views.IsNull() && views.HasData())
                     {
@@ -466,6 +512,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSets(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSetWithTemplate(
+    const ApiCallInfo&               call_info,
     format::HandleId                 device,
     format::HandleId                 descriptorSet,
     format::HandleId                 descriptorUpdateTemplate,
@@ -477,6 +524,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSetWithTemp
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkCmdPushDescriptorSetWithTemplateKHR(
+    const ApiCallInfo&               call_info,
     format::HandleId                 commandBuffer,
     format::HandleId                 descriptorUpdateTemplate,
     format::HandleId                 layout,
@@ -490,6 +538,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkCmdPushDescriptorSetWithTem
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSetWithTemplateKHR(
+    const ApiCallInfo&               call_info,
     format::HandleId                 device,
     format::HandleId                 descriptorSet,
     format::HandleId                 descriptorUpdateTemplate,
@@ -501,6 +550,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkUpdateDescriptorSetWithTemp
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkDestroyCommandPool(
+    const ApiCallInfo&                                   call_info,
     format::HandleId                                     device,
     format::HandleId                                     commandPool,
     StructPointerDecoder<Decoded_VkAllocationCallbacks>* pAllocator)
@@ -511,7 +561,8 @@ void VulkanReferencedResourceConsumerBase::Process_vkDestroyCommandPool(
     table_.ClearUsers(commandPool);
 }
 
-void VulkanReferencedResourceConsumerBase::Process_vkResetCommandPool(VkResult                returnValue,
+void VulkanReferencedResourceConsumerBase::Process_vkResetCommandPool(const ApiCallInfo&      call_info,
+                                                                      VkResult                returnValue,
                                                                       format::HandleId        device,
                                                                       format::HandleId        commandPool,
                                                                       VkCommandPoolResetFlags flags)
@@ -524,6 +575,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkResetCommandPool(VkResult
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkAllocateCommandBuffers(
+    const ApiCallInfo&                                         call_info,
     VkResult                                                   returnValue,
     format::HandleId                                           device,
     StructPointerDecoder<Decoded_VkCommandBufferAllocateInfo>* pAllocateInfo,
@@ -551,6 +603,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkAllocateCommandBuffers(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkFreeCommandBuffers(
+    const ApiCallInfo&                     call_info,
     format::HandleId                       device,
     format::HandleId                       commandPool,
     uint32_t                               commandBufferCount,
@@ -573,6 +626,7 @@ void VulkanReferencedResourceConsumerBase::Process_vkFreeCommandBuffers(
 }
 
 void VulkanReferencedResourceConsumerBase::Process_vkBeginCommandBuffer(
+    const ApiCallInfo&                                      call_info,
     VkResult                                                returnValue,
     format::HandleId                                        commandBuffer,
     StructPointerDecoder<Decoded_VkCommandBufferBeginInfo>* pBeginInfo)
@@ -583,7 +637,8 @@ void VulkanReferencedResourceConsumerBase::Process_vkBeginCommandBuffer(
     table_.ResetUser(commandBuffer);
 }
 
-void VulkanReferencedResourceConsumerBase::Process_vkResetCommandBuffer(VkResult                  returnValue,
+void VulkanReferencedResourceConsumerBase::Process_vkResetCommandBuffer(const ApiCallInfo&        call_info,
+                                                                        VkResult                  returnValue,
                                                                         format::HandleId          commandBuffer,
                                                                         VkCommandBufferResetFlags flags)
 {
