@@ -1,6 +1,7 @@
 /*
 ** Copyright (c) 2018-2020 Valve Corporation
 ** Copyright (c) 2018-2020 LunarG, Inc.
+** Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -62,6 +63,7 @@ class ParameterEncoder
     void EncodeInt64Value(int64_t value)                                                                              { EncodeValue(value); }
     void EncodeUInt64Value(uint64_t value)                                                                            { EncodeValue(value); }
     void EncodeFloatValue(float value)                                                                                { EncodeValue(value); }
+    void EncodeDoubleValue(double value)                                                                              { EncodeValue(value); }
     void EncodeVkBool32Value(VkBool32 value)                                                                          { EncodeValue(value); }
     void EncodeVkSampleMaskValue(VkSampleMask value)                                                                  { EncodeValue(static_cast<format::SampleMaskEncodeType>(value)); }
     void EncodeVkDeviceSizeValue(VkDeviceSize value)                                                                  { EncodeValue(static_cast<format::DeviceSizeEncodeType>(value)); }
@@ -75,8 +77,8 @@ class ParameterEncoder
     template<typename T>
     void EncodeFunctionPtr(T value)                                                                                   { EncodeValue(reinterpret_cast<format::AddressEncodeType>(value)); }
 
-    template<typename T>
-    void EncodeHandleValue(T value)                                                                                   { EncodeHandleIdValue(GetWrappedId(value)); }
+    template<typename Wrapper>
+    void EncodeHandleValue(typename Wrapper::HandleType value)                                                        { EncodeHandleIdValue(GetWrappedId<Wrapper>(value)); }
     template<typename T>
     void EncodeEnumValue(T value)                                                                                     { EncodeValue(static_cast<format::EnumEncodeType>(value)); }
     template<typename T>
@@ -109,8 +111,8 @@ class ParameterEncoder
     template<typename T>
     void EncodeVoidPtrPtr(const T* const* ptr, bool omit_data = false, bool omit_addr = false)                        { EncodePointerConverted<format::AddressEncodeType>(ptr, omit_data, omit_addr); }
 
-    template<typename T>
-    void EncodeHandlePtr(const T* ptr, bool omit_data = false, bool omit_addr = false)                                { EncodeWrappedHandlePointer(ptr, omit_data, omit_addr); }
+    template<typename Wrapper>
+    void EncodeHandlePtr(const typename Wrapper::HandleType* ptr, bool omit_data = false, bool omit_addr = false)     { EncodeWrappedHandlePointer<Wrapper>(ptr, omit_data, omit_addr); }
     template<typename T>
     void EncodeEnumPtr(const T* ptr, bool omit_data = false, bool omit_addr = false)                                  { EncodePointerConverted<format::EnumEncodeType>(ptr, omit_data, omit_addr); }
     template<typename T>
@@ -119,6 +121,7 @@ class ParameterEncoder
     void EncodeFlags64Ptr(const T* ptr, bool omit_data = false, bool omit_addr = false)                               { EncodePointerConverted<format::Flags64EncodeType>(ptr, omit_data, omit_addr); }
 
     // Arrays
+    void EncodeInt8Array(const int8_t* arr, size_t len, bool omit_data = false, bool omit_addr = false)               { EncodeArray(arr, len, omit_data, omit_addr); }
     void EncodeInt16Array(const int16_t* arr, size_t len, bool omit_data = false, bool omit_addr = false)             { EncodeArray(arr, len, omit_data, omit_addr); }
     void EncodeUInt16Array(const uint16_t* arr, size_t len, bool omit_data = false, bool omit_addr = false)           { EncodeArray(arr, len, omit_data, omit_addr); }
     void EncodeInt32Array(const int32_t* arr, size_t len, bool omit_data = false, bool omit_addr = false)             { EncodeArray(arr, len, omit_data, omit_addr); }
@@ -137,8 +140,8 @@ class ParameterEncoder
     void EncodeUInt8Array(const void* arr, size_t len, bool omit_data = false, bool omit_addr = false)                { EncodeArray(reinterpret_cast<const uint8_t*>(arr), len, omit_data, omit_addr); }
     void EncodeVoidArray(const void* arr, size_t len, bool omit_data = false, bool omit_addr = false)                 { EncodeArray(reinterpret_cast<const uint8_t*>(arr), len, omit_data, omit_addr); }
 
-    template<typename T>
-    void EncodeHandleArray(const T* arr, size_t len, bool omit_data = false, bool omit_addr = false)                  { EncodeWrappedHandleArray(arr, len, omit_data, omit_addr); }
+    template<typename Wrapper>
+    void EncodeHandleArray(const typename Wrapper::HandleType* arr, size_t len, bool omit_data = false, bool omit_addr = false) { EncodeWrappedHandleArray<Wrapper>(arr, len, omit_data, omit_addr); }
     template<typename T>
     void EncodeEnumArray(const T* arr, size_t len, bool omit_data = false, bool omit_addr = false)                    { EncodeArrayConverted<format::EnumEncodeType>(arr, len, omit_data, omit_addr); }
     template<typename T>
@@ -157,6 +160,12 @@ class ParameterEncoder
 
     template <size_t N, size_t M>
     void EncodeFloat2DMatrix(const float (&arr)[N][M], size_t n, size_t m, bool omit_data = false, bool omit_addr = false) { assert((N == n) && (M == m)); EncodeArray(reinterpret_cast<const float*>(arr), n * m, omit_data, omit_addr); }
+    template <size_t N, size_t M>
+    void EncodeUInt82DMatrix(const uint8_t (&arr)[N][M], size_t n, size_t m, bool omit_data = false, bool omit_addr = false) { assert((N == n) && (M == m)); EncodeArray(reinterpret_cast<const uint8_t*>(arr), n * m, omit_data, omit_addr); }
+    template <size_t N, size_t M>
+    void EncodeInt82DMatrix(const int8_t (&arr)[N][M], size_t n, size_t m, bool omit_data = false, bool omit_addr = false) { assert((N == n) && (M == m)); EncodeArray(reinterpret_cast<const int8_t*>(arr), n * m, omit_data, omit_addr); }
+    template <size_t N, size_t M>
+    void EncodeUInt162DMatrix(const uint16_t (&arr)[N][M], size_t n, size_t m, bool omit_data = false, bool omit_addr = false) { assert((N == n) && (M == m)); EncodeArray(reinterpret_cast<const uint16_t*>(arr), n * m, omit_data, omit_addr); }
 
     // clang-format on
 
@@ -376,8 +385,9 @@ class ParameterEncoder
         }
     }
 
-    template <typename SrcT>
-    void EncodeWrappedHandlePointer(const SrcT* ptr, bool omit_data = false, bool omit_addr = false)
+    template <typename Wrapper>
+    void
+    EncodeWrappedHandlePointer(const typename Wrapper::HandleType* ptr, bool omit_data = false, bool omit_addr = false)
     {
         uint32_t pointer_attrib =
             format::PointerAttributes::kIsSingle | GetPointerAttributeMask(ptr, omit_data, omit_addr);
@@ -393,7 +403,7 @@ class ParameterEncoder
 
             if ((pointer_attrib & format::PointerAttributes::kHasData) == format::PointerAttributes::kHasData)
             {
-                EncodeHandleValue(*ptr);
+                EncodeHandleValue<Wrapper>(*ptr);
             }
         }
     }
@@ -464,8 +474,11 @@ class ParameterEncoder
         EncodeArray(arr, len, omit_data, omit_addr);
     }
 
-    template <typename SrcT>
-    void EncodeWrappedHandleArray(const SrcT* arr, size_t len, bool omit_data = false, bool omit_addr = false)
+    template <typename Wrapper>
+    void EncodeWrappedHandleArray(const typename Wrapper::HandleType* arr,
+                                  size_t                              len,
+                                  bool                                omit_data = false,
+                                  bool                                omit_addr = false)
     {
         uint32_t pointer_attrib =
             format::PointerAttributes::kIsArray | GetPointerAttributeMask(arr, omit_data, omit_addr);
@@ -486,7 +499,7 @@ class ParameterEncoder
             {
                 for (size_t i = 0; i < len; ++i)
                 {
-                    EncodeHandleValue(arr[i]);
+                    EncodeHandleValue<Wrapper>(arr[i]);
                 }
             }
         }
